"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var auto_complete_event_1 = require("../event/auto-complete-event");
var rule_1 = require("../util/rule");
var AutoCompletionHandler = /** @class */ (function () {
    function AutoCompletionHandler(editor) {
        this.editor = editor;
    }
    AutoCompletionHandler.prototype.processEvents = function () {
        var _this = this;
        this.editor.addChangeListener(function (event) {
            if (event.hasInsertEditChange()) {
                var rule = _this.getRuleHint();
                if (rule) {
                    console.log(event);
                    console.log(rule);
                    var stack_1 = [rule];
                    while (stack_1.length > 0) {
                        var root = stack_1.pop();
                        var autoCompleteEvent = new auto_complete_event_1.AutoCompleteEvent(root, _this.editor);
                        _this.editor.triggerAutoCompletionEvent(autoCompleteEvent);
                        if (autoCompleteEvent.allowChildrenToConsumeEvent) {
                            root.getChildren()
                                .forEach(function (child) {
                                stack_1.push(child);
                            });
                        }
                    }
                }
            }
            else {
                _this.editor.clearAllCompletions();
            }
        });
    };
    AutoCompletionHandler.prototype.getErrorRuleHint = function () {
        var parser = this.editor.getParser();
        if (parser.hasErrors()) {
            var cursorPosition = this.editor.getCursorPosition();
            var colStart = Math.max(0, cursorPosition.column - 1);
            for (var i = cursorPosition.line; i >= 0; i--) {
                for (var j = colStart; j >= 0; j--) {
                    var rule = parser.getErrorRuleAt(j, i);
                    if (rule) {
                        return rule;
                    }
                }
                colStart = Math.max(0, parser.getColumnCount(i - 1) - 1);
            }
        }
        return undefined;
    };
    AutoCompletionHandler.prototype.getRuleHint = function () {
        var errorRuleHint = this.getErrorRuleHint();
        var cursorPosition = this.editor.getCursorPosition();
        var parser = this.editor.getParser();
        var ruleLookup = null;
        var colStart = Math.max(0, cursorPosition.column - 1);
        for (var i = cursorPosition.line; i >= 0; i--) {
            for (var j = colStart; j >= 0; j--) {
                var rule = parser.getRuleAt(j, i);
                if (rule) {
                    ruleLookup = rule;
                    break;
                }
            }
            if (ruleLookup) {
                break;
            }
            colStart = Math.max(0, parser.getColumnCount(i - 1) - 1);
        }
        if (errorRuleHint && ruleLookup) {
            if (rule_1.isRuleParent(errorRuleHint.getRule(), ruleLookup.getRule())) {
                return ruleLookup;
            }
            else {
                return errorRuleHint;
            }
            // const errorPriority = this.computeRulePriority(errorRuleHint);
            // const rulePriority = this.computeRulePriority(ruleLookup);
            // if (errorPriority > rulePriority) {
            //     return errorRuleHint;
            // } else {
            //     return ruleLookup;
            // }
        }
        else if (errorRuleHint) {
            return errorRuleHint;
        }
        else if (ruleLookup) {
            return ruleLookup;
        }
        return undefined;
    };
    AutoCompletionHandler.prototype.computeRulePriority = function (ruleWrapper) {
        var rule = ruleWrapper.getRule();
        return rule.depth() * rule.invokingState;
    };
    return AutoCompletionHandler;
}());
exports.AutoCompletionHandler = AutoCompletionHandler;
//# sourceMappingURL=auto-completion-handler.js.map