"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var expressive_template_1 = require("expressive-template");
var CompletionTemplateHandler = /** @class */ (function () {
    function CompletionTemplateHandler(editor) {
        this.editor = editor;
    }
    CompletionTemplateHandler.prototype.processCompletion = function (completion) {
        var _this = this;
        completion = _.clone(completion);
        if (_.isNil(completion.placeholderVariables)) {
            completion.placeholderVariables = {};
        }
        var text = completion.text;
        var ctx = completion.placeholderVariables;
        var templateRes = expressive_template_1.renderTemplateExpressions(text, function (varName) {
            if (ctx[varName]) {
                var placeholderVar = ctx[varName];
                if (placeholderVar.defaultValue) {
                    return placeholderVar.defaultValue;
                }
            }
            return ' ';
        });
        completion.text = templateRes.text;
        completion.placeholderFactoryMethods = {};
        completion.placeholders = {};
        templateRes.expressions
            .forEach(function (expr) {
            var range = expr.getCurrentRange();
            var newRange = [];
            newRange[0] = {
                column: range[0].column + completion.range[0].column,
                line: range[0].line + completion.range[0].line
            };
            newRange[1] = {
                column: range[1].column + completion.range[0].column,
                line: range[1].line + completion.range[0].line
            };
            var name = expr.getVariableName();
            completion.placeholderFactoryMethods[name] = function () {
                var placeholder = _this.editor.createPlaceholder(newRange);
                var placeholderVar = ctx[name];
                placeholderVar.enableDefaultKeyEventHandlers = (_.isNil(placeholderVar.enableDefaultKeyEventHandlers)) ? true : placeholderVar.enableDefaultKeyEventHandlers;
                placeholderVar.applyFocus = (_.isNil(placeholderVar.applyFocus)) ? true : placeholderVar.applyFocus;
                if (placeholderVar && placeholderVar.placeHolderText) {
                    placeholder.setPlaceHolderText(ctx[name].placeHolderText);
                }
                else {
                    placeholder.setPlaceHolderText(name);
                }
                if (placeholderVar.enableDefaultKeyEventHandlers) {
                    placeholder.enableDefaultKeyEventHandlers();
                }
                if (placeholderVar.applyFocus) {
                    placeholder.focus();
                }
                return placeholder;
            };
        });
        return completion;
    };
    return CompletionTemplateHandler;
}());
exports.CompletionTemplateHandler = CompletionTemplateHandler;
//# sourceMappingURL=completion-template-handler.js.map