"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//
// CodeMirror addons
//
require('codemirror/addon/lint/lint');
require('codemirror/addon/selection/active-line');
require('codemirror/addon/display/placeholder');
require('codemirror/addon/runmode/runmode');
require('codemirror/keymap/vim');
require('codemirror/addon/display/autorefresh');
exports.ACTIVE_HINT_CLASS = 'CodeMirror-hint-active';
exports.HINT_CLASS = 'CodeMirror-hint';
exports.HINTS_CLASS = 'CodeMirror-hints';
var CodeMirror = require("codemirror");
var _ = require("lodash");
var index_1 = require("rxjs/index");
var codemirror_rule_decoration_1 = require("./decoration/codemirror-rule-decoration");
var codemirror_token_decoration_1 = require("./decoration/codemirror-token-decoration");
var operators_1 = require("rxjs/internal/operators");
var auto_completion_handler_1 = require("../../handler/auto-completion-handler");
var generic_completion_popup_1 = require("../../completion/generic-completion-popup");
var codemirror_change_event_1 = require("./event/codemirror-change-event");
var codemirror_placeholder_1 = require("./placeholder/codemirror-placeholder");
var codemirror_bookmark_1 = require("./bookmark/codemirror-bookmark");
var CodeMirrorEditor = /** @class */ (function () {
    function CodeMirrorEditor(parser, domContainer) {
        var _this = this;
        this.parser = parser;
        this.domContainer = domContainer;
        var mode = _.uniqueId('antlrGrammarMode');
        this.displayDecorations = true;
        this.customErrors = new Set();
        this.changeSubject = new index_1.Subject();
        this.validators = new Set();
        this.setValueEvent = false;
        this.ruleErrorMessageHandlers = new Map();
        this.autoCompleteSubject = new index_1.Subject();
        this.defaultTokenStyles = {};
        this.defaultRuleStyles = {};
        this.ruleDecorations = new Map();
        this.tokenDecorations = new Map();
        this.autoCompletionHandler = new auto_completion_handler_1.AutoCompletionHandler(this);
        this.placeholdersRendered = new Set();
        this.bookmarkDecorations = new Set();
        this.selections = new Set();
        if (this.domContainer === undefined) {
            this.domContainer = document.createElement('div');
        }
        this.domContainer.classList.add('antlr-editor');
        this.hintContainer = this.domContainer;
        this.editorImplementation = CodeMirror(this.domContainer, {});
        this.editorImplementation.getDoc().getMode().name = mode;
        this.editorImplementation.setOption('autoRefresh', true);
        this.defaultCursorBlinkingRate = this.editorImplementation.getOption('cursorBlinkRate');
        this.attachBasicEvents();
        this.autoCompletionHandler.processEvents();
        this.updateCursorPosition();
        CodeMirror.registerHelper('lint', mode, function () {
            if (!_this.displaySingleError) {
                var parserErrors = _this.getEditorErrors();
                return parserErrors.map(function (err) { return _this.mapParseErrorToCodeMirrorError(err); });
            }
            else {
                if (_this.hasErrors()) {
                    return [_this.mapParseErrorToCodeMirrorError(_this.getRelevantEditorError())];
                }
                return [];
            }
        });
        parser.addParserStartListener(function () {
            Array.from(_this.tokenDecorations.values())
                .forEach(function (decoration) { return decoration.remove(); });
            Array.from(_this.ruleDecorations.values())
                .forEach(function (decoration) { return decoration.remove(); });
            Array.from(_this.bookmarkDecorations.values())
                .forEach(function (decoration) { return decoration.remove(); });
            _this.selections.clear();
            _this.ruleDecorations.clear();
            _this.bookmarkDecorations.clear();
            _this.tokenDecorations.clear();
            _this.placeholdersRendered.clear();
            _this.customErrors.clear();
        });
        parser.addParserCompleteListener(function () {
            _this.clearAllCompletions();
            parser.getAllRules().filter(function (rule) { return rule.exists(); })
                .map(function (rule) {
                var ruleName = rule.getName();
                if (!_.isNil(_this.defaultRuleStyles[ruleName])) {
                    var style = _this.defaultRuleStyles[ruleName];
                    _this.styleRule(rule, style);
                }
                else {
                    _this.styleRule(rule);
                }
            });
            parser.getAllTokens().filter(function (token) { return token.exists(); })
                .forEach(function (token) {
                var name = token.getName() ? token.getName() : token.getText();
                var style = _this.defaultTokenStyles[name];
                _this.styleToken(token, style);
            });
            if (_this.lastChangeEvent) {
                _this.changeSubject.next(_this.lastChangeEvent);
                _this.lastChangeEvent = undefined;
            }
        });
    }
    CodeMirrorEditor.prototype.setHintContainer = function (el) {
        this.hintContainer = el;
    };
    CodeMirrorEditor.prototype.replaceRange = function (range, text) {
        var newRange = this.parser.replaceRange(range, text);
        this.cursorPosition = newRange[1];
        this.setValueEvent = true;
        var change = {
            from: { ch: newRange[0].column, line: newRange[0].line },
            to: { ch: newRange[0].column, line: newRange[0].line },
            text: [text],
            removed: [],
            origin: 'setValue'
        };
        this.lastChangeEvent = new codemirror_change_event_1.CodeMirrorChangeEvent(this, [change]);
        this.editorImplementation.getDoc().setCursor({ ch: this.cursorPosition.column, line: this.cursorPosition.line });
        this.editorImplementation.setValue(this.parser.getText());
        this.parser.reparse();
        this.editorImplementation.refresh();
        return newRange;
    };
    CodeMirrorEditor.prototype.setText = function (text) {
        this.updateCursorPosition();
        this.setValueEvent = true;
        this.editorImplementation.setValue(this.parser.getText());
        this.parser.parse(text);
    };
    CodeMirrorEditor.prototype.update = function () {
        this.updateCursorPosition();
        this.setValueEvent = true;
        this.editorImplementation.setValue(this.parser.getText());
        this.parser.reparse();
    };
    CodeMirrorEditor.prototype.getText = function () {
        return this.parser.getText();
    };
    CodeMirrorEditor.prototype.getParser = function () {
        return this.parser;
    };
    CodeMirrorEditor.prototype.setParser = function (parser) {
        this.parser = parser;
    };
    CodeMirrorEditor.prototype.addKeyMapping = function (mapping) {
        this.editorImplementation.addKeyMap(mapping);
    };
    CodeMirrorEditor.prototype.removeKeyMapping = function (mapping) {
        this.editorImplementation.removeKeyMap(mapping);
    };
    CodeMirrorEditor.prototype.focus = function () {
        this.editorImplementation.focus();
    };
    CodeMirrorEditor.prototype.createRuleDecoration = function (rule, domElement) {
        var decoration = new codemirror_rule_decoration_1.CodeMirrorRuleDecoration(this, rule, domElement);
        this.ruleDecorations.set(rule.getRule(), decoration);
        return decoration;
    };
    CodeMirrorEditor.prototype.getRuleDecoration = function (rule) {
        return this.ruleDecorations.get(rule.getRule());
    };
    CodeMirrorEditor.prototype.createTokenDecoration = function (token, domElement) {
        var decoration = new codemirror_token_decoration_1.CodeMirrorTokenDecoration(this, token, domElement);
        this.tokenDecorations.set(token.getToken(), decoration);
        return decoration;
    };
    CodeMirrorEditor.prototype.getTokenDecoration = function (token) {
        return this.tokenDecorations.get(token.getToken());
    };
    CodeMirrorEditor.prototype.getDomElement = function () {
        return this.domContainer;
    };
    CodeMirrorEditor.prototype.setTheme = function (theme) {
        this.editorImplementation.setOption('theme', theme);
    };
    CodeMirrorEditor.prototype.getTheme = function () {
        return this.editorImplementation.getOption('theme');
    };
    CodeMirrorEditor.prototype.setDefaultTokenStyles = function (styles) {
        this.defaultTokenStyles = styles;
    };
    CodeMirrorEditor.prototype.setDefaultRuleStyles = function (styles) {
        this.defaultRuleStyles = styles;
    };
    CodeMirrorEditor.prototype.updateCursorPosition = function () {
        var cursor = this.editorImplementation.getDoc().getCursor();
        this.cursorPosition = { column: cursor.ch, line: cursor.line };
    };
    CodeMirrorEditor.prototype.setDisplayEditorErrors = function (display) {
        this.editorImplementation.setOption('lint', display);
    };
    CodeMirrorEditor.prototype.getDisplayEditorErrors = function () {
        return this.editorImplementation.getOption('lint');
    };
    CodeMirrorEditor.prototype.setEditorPlaceholderText = function (text) {
        this.editorImplementation.setOption('placeholder', text);
    };
    CodeMirrorEditor.prototype.setReadOnly = function (readOnly) {
        if (readOnly) {
            this.editorImplementation.setOption('cursorBlinkRate', -100);
            this.editorImplementation.setOption('readOnly', readOnly);
        }
        else {
            this.editorImplementation.setOption('cursorBlinkRate', this.defaultCursorBlinkingRate);
            this.editorImplementation.setOption('readOnly', readOnly);
        }
    };
    CodeMirrorEditor.prototype.getReadOnly = function () {
        return !!this.editorImplementation.getOption('readOnly');
    };
    CodeMirrorEditor.prototype.setStyleActiveLine = function (styleActiveLine) {
        this.editorImplementation.setOption('styleActiveLine', styleActiveLine);
    };
    CodeMirrorEditor.prototype.getStyleActiveLine = function () {
        return !!this.editorImplementation.getOption('styleActiveLine');
    };
    CodeMirrorEditor.prototype.setLineWrapping = function (lineWrapping) {
        this.editorImplementation.setOption('lineWrapping', lineWrapping);
    };
    CodeMirrorEditor.prototype.getLineWrapping = function () {
        return !!this.editorImplementation.getOption('lineWrapping');
    };
    CodeMirrorEditor.prototype.getEditorPlaceholderText = function () {
        return this.editorImplementation.getOption('placeholder');
    };
    CodeMirrorEditor.prototype.setDisplayOnlyRelevantEditorErrors = function (display) {
        this.displaySingleError = display;
    };
    CodeMirrorEditor.prototype.getDisplayOnlyRelevantEditorErrors = function () {
        return this.displaySingleError;
    };
    CodeMirrorEditor.prototype.setShowLineNumbers = function (show) {
        this.editorImplementation.setOption('lineNumbers', show);
    };
    CodeMirrorEditor.prototype.getShowLineNumbers = function () {
        return this.editorImplementation.getOption('lineNumbers');
    };
    CodeMirrorEditor.prototype.addStyleToRule = function (rule, css) {
        var ruleRange = rule.getRange();
        this.editorImplementation.getDoc().markText({ ch: ruleRange[0].column, line: ruleRange[0].line }, {
            ch: ruleRange[1].column,
            line: ruleRange[1].line
        }, { css: css });
    };
    CodeMirrorEditor.prototype.addClassToRule = function (rule, cssClass) {
        var ruleRange = rule.getRange();
        this.editorImplementation.getDoc().markText({ ch: ruleRange[0].column, line: ruleRange[0].line }, {
            ch: ruleRange[1].column,
            line: ruleRange[1].line
        }, { className: cssClass });
    };
    CodeMirrorEditor.prototype.addStyleToToken = function (token, css) {
        var tokenRange = token.getRange();
        this.editorImplementation.getDoc().markText({ ch: tokenRange[0].column, line: tokenRange[0].line }, {
            ch: tokenRange[1].column,
            line: tokenRange[1].line
        }, { css: css });
    };
    CodeMirrorEditor.prototype.addClassToToken = function (token, cssClass) {
        var tokenRange = token.getRange();
        this.editorImplementation.getDoc().markText({ ch: tokenRange[0].column, line: tokenRange[0].line }, {
            ch: tokenRange[1].column,
            line: tokenRange[1].line
        }, { className: cssClass });
    };
    CodeMirrorEditor.prototype.addAutoCompleteListener = function (rule, listener) {
        this.autoCompleteSubject
            .pipe(operators_1.filter(function (event) { return event.rule.getName() === rule; }))
            .subscribe(listener);
    };
    CodeMirrorEditor.prototype.addChangeListener = function (listener) {
        this.changeSubject.asObservable().subscribe(listener);
    };
    CodeMirrorEditor.prototype.setCursorPosition = function (position) {
        this.cursorPosition = position;
        this.editorImplementation
            .getDoc()
            .setCursor({ ch: position.column, line: position.line });
    };
    CodeMirrorEditor.prototype.getCursorPosition = function () {
        return this.cursorPosition;
    };
    CodeMirrorEditor.prototype.defineErrorMessage = function (ruleOrMessage, handler) {
        this.ruleErrorMessageHandlers.set(ruleOrMessage, handler);
    };
    CodeMirrorEditor.prototype.getEditorErrors = function () {
        var _this = this;
        var errors = _.concat(Array.from(this.customErrors), this.parser.getErrors());
        return errors.map(function (err) {
            var newErr = _.clone(err);
            if (_this.ruleErrorMessageHandlers.has(err.message)) {
                var func = _this.ruleErrorMessageHandlers.get(err.message);
                var message = func(err);
                newErr.message = message;
            }
            else if (err.ruleWrapper && err.ruleWrapper) {
                if (_this.ruleErrorMessageHandlers.has(err.ruleWrapper.getName())) {
                    var func = _this.ruleErrorMessageHandlers.get(err.ruleWrapper.getName());
                    var message = func(err);
                    newErr.message = message;
                }
            }
            return newErr;
        });
    };
    CodeMirrorEditor.prototype.getRelevantEditorError = function () {
        var errors = this.getEditorErrors();
        if (errors.length > 0) {
            return _.last(errors);
        }
        return undefined;
    };
    CodeMirrorEditor.prototype.setDisplayDecorations = function (display) {
        this.displayDecorations = display;
        if (display) {
            this.showAllDecorations();
        }
        else {
            this.hideAllDecorations();
        }
    };
    CodeMirrorEditor.prototype.getDisplayDecorations = function () {
        return this.displayDecorations;
    };
    CodeMirrorEditor.prototype.showCompletions = function (completions) {
        var _this = this;
        this.clearAllCompletions();
        this.addKeyMapping(this.defaultHintMapping);
        var popup = new generic_completion_popup_1.GenericCompletionPopup(this.hintContainer, this);
        this.currentCompletionPopup = popup;
        setTimeout(function () {
            var cursorEl = _this.domContainer.getElementsByClassName('CodeMirror-cursor').item(0);
            popup.singleCompletionCssClass = exports.HINT_CLASS;
            popup.completionsCssClass = exports.HINTS_CLASS;
            popup.activeCompletionCssClass = exports.ACTIVE_HINT_CLASS;
            popup.showHints(cursorEl, completions);
        });
        return popup;
    };
    CodeMirrorEditor.prototype.createRuleCompletion = function (baseCompletion, rule) {
        var newCompletion = _.clone(baseCompletion);
        newCompletion.range = rule.getRange();
        return newCompletion;
    };
    CodeMirrorEditor.prototype.triggerAutoCompletionEvent = function (event) {
        this.autoCompleteSubject.next(event);
    };
    CodeMirrorEditor.prototype.hasRenderedPlaceholders = function () {
        var list = Array.from(this.placeholdersRendered).filter(function (placeholder) { return placeholder.exists(); });
        return list.length > 0;
    };
    CodeMirrorEditor.prototype.createPlaceholder = function (range) {
        var el = document.createElement('span');
        var placeholder = new codemirror_placeholder_1.CodeMirrorPlaceholder(this, el, range);
        this.clearAllCompletions();
        this.placeholdersRendered.add(placeholder);
        return placeholder;
    };
    CodeMirrorEditor.prototype.createRulePlaceholder = function (rule) {
        var placeHolder = this.createPlaceholder(rule.getRange());
        placeHolder.setPlaceHolderText(rule.getName());
        this.placeholdersRendered.add(placeHolder);
        return placeHolder;
    };
    CodeMirrorEditor.prototype.createBookmarkDecoration = function (start, dom, insertLeft) {
        var bookmark = new codemirror_bookmark_1.CodeMirrorBookmark(this, start, dom, insertLeft);
        this.bookmarkDecorations.add(bookmark);
        return bookmark;
    };
    CodeMirrorEditor.prototype.hasSelections = function () {
        return this.editorImplementation.getDoc().somethingSelected();
    };
    CodeMirrorEditor.prototype.addTokenSelection = function (token) {
        this.addSelection(token.getRange());
    };
    CodeMirrorEditor.prototype.addRuleSelection = function (rule) {
        this.addSelection(rule.getRange());
    };
    CodeMirrorEditor.prototype.addSelection = function (range) {
        var anchor = { ch: range[0].column, line: range[0].line };
        var head = { ch: range[1].column, line: range[1].line };
        this.selections.add(range);
        this.editorImplementation.getDoc().addSelection(anchor, head);
    };
    CodeMirrorEditor.prototype.getSelections = function () {
        return Array.from(this.selections);
    };
    CodeMirrorEditor.prototype.setEnableVim = function (enable) {
        if (enable) {
            this.editorImplementation.setOption('keyMap', 'vim');
        }
        else {
            this.editorImplementation.setOption('keyMap', 'default');
        }
    };
    CodeMirrorEditor.prototype.getEnableVim = function () {
        return this.editorImplementation.getOption('keyMap') === 'vim';
    };
    CodeMirrorEditor.prototype.getRuleByName = function (ruleName) {
        return this.parser.findRuleByName(ruleName);
    };
    CodeMirrorEditor.prototype.getRulesByName = function (ruleName) {
        return this.parser.findRulesByName(ruleName);
    };
    CodeMirrorEditor.prototype.getAllRules = function () {
        return this.parser.getAllRules();
    };
    CodeMirrorEditor.prototype.getAllTokens = function () {
        return this.parser.getAllTokens();
    };
    CodeMirrorEditor.prototype.addEditorValidator = function (validator) {
        this.validators.add(validator);
    };
    CodeMirrorEditor.prototype.hasErrors = function () {
        return this.getEditorErrors().length > 0;
    };
    CodeMirrorEditor.prototype.createRuleError = function (rule) {
        if (rule) {
            return this.parser.createRuleError(rule.getRule());
        }
        else {
            return {
                start: { column: 0, line: 0 },
                end: { column: 0, line: 0 },
                message: '',
                exception: undefined,
                severity: 'error',
                rule: undefined,
                ruleWrapper: undefined
            };
        }
    };
    CodeMirrorEditor.prototype.validate = function () {
        var _this = this;
        Array.from(this.validators).forEach(function (validator) {
            _this.getAllRules().forEach(function (rule) {
                var res = validator(rule);
                if (!_.isNil(res)) {
                    _this.customErrors.add(res);
                }
            });
        });
        this.editorImplementation.performLint();
    };
    CodeMirrorEditor.prototype.clearAllCompletions = function () {
        if (!_.isNil(this.currentCompletionPopup)) {
            this.currentCompletionPopup.removeCompletions();
            this.removeKeyMapping(this.defaultHintMapping);
        }
    };
    CodeMirrorEditor.prototype.styleToken = function (token, styleClass) {
        var tokenRange = token.getRange();
        var tokenClass = (token.getName()) ? "antlr-token-" + token.getName() : 'antlr-token';
        this.editorImplementation.getDoc().markText({ ch: tokenRange[0].column, line: tokenRange[0].line }, {
            ch: tokenRange[1].column,
            line: tokenRange[1].line
        }, { className: tokenClass + " " + (styleClass ? styleClass : '') });
    };
    CodeMirrorEditor.prototype.styleRule = function (rule, styleClass) {
        var ruleRange = rule.getRange();
        var ruleClass = "antlr-rule-" + rule.getName();
        this.editorImplementation.getDoc().markText({ ch: ruleRange[0].column, line: ruleRange[0].line }, {
            ch: ruleRange[1].column,
            line: ruleRange[1].line
        }, { className: ruleClass + " " + (styleClass ? styleClass : '') });
    };
    CodeMirrorEditor.prototype.mapParseErrorToCodeMirrorError = function (err) {
        return {
            from: { ch: err.start.column, line: err.start.line },
            to: { ch: err.end.column, line: err.end.line },
            message: err.message,
            severity: err.severity
        };
    };
    CodeMirrorEditor.prototype.attachBasicEvents = function () {
        var _this = this;
        this.defaultHintMapping = {
            Up: function () { return _this.currentCompletionPopup.choosePrevCompletion(); },
            Enter: function () { return _this.currentCompletionPopup.select(); },
            Esc: function () { return _this.currentCompletionPopup.cancel(); },
            Down: function () { return _this.currentCompletionPopup.chooseNextCompletion(); },
        };
        window.addEventListener('blur', function () { return _this.clearAllCompletions(); });
        this.editorImplementation.on('changes', function (___, changes) {
            _this.lastChangeEvent = new codemirror_change_event_1.CodeMirrorChangeEvent(_this, changes);
            if (_this.setValueEvent) {
                //
                // Set the cursor the the last know position found.
                // If we see a direct change made to the editor.
                //
                // Typically done after the parsing  is complete.
                //
                _this.editorImplementation.getDoc()
                    .setCursor({
                    ch: _this.cursorPosition.column,
                    line: _this.cursorPosition.line
                });
                _this.setValueEvent = false;
            }
            else if (changes.length >= 1) {
                ///
                // Only need to change the parser text on a editor change.
                //
                _this.parser.parse(_this.editorImplementation.getValue());
            }
        });
        this.editorImplementation.on('cursorActivity', function () {
            if (!_this.setValueEvent) {
                console.log(_this.editorImplementation.getDoc().getCursor());
                _this.updateCursorPosition();
                return;
            }
        });
    };
    CodeMirrorEditor.prototype.hideAllDecorations = function () {
        Array.from(this.ruleDecorations.values()).forEach(function (decoration) {
            decoration.hide();
        });
        Array.from(this.tokenDecorations.values()).forEach(function (decoration) {
            decoration.hide();
        });
    };
    CodeMirrorEditor.prototype.showAllDecorations = function () {
        Array.from(this.ruleDecorations.values()).forEach(function (decoration) {
            decoration.show();
        });
        Array.from(this.tokenDecorations.values()).forEach(function (decoration) {
            decoration.show();
        });
    };
    return CodeMirrorEditor;
}());
exports.CodeMirrorEditor = CodeMirrorEditor;
//# sourceMappingURL=codemirror-editor.js.map